pub mod common;

use cuddle_please_misc::VcsClient;
use tracing_test::traced_test;

use crate::common::get_test_data_path;

#[test]
#[traced_test]
fn test_vcs_get_noop() {
    let noop = VcsClient::new_noop();
    assert!(matches!(noop, VcsClient::Noop {}));
}

#[test]
#[traced_test]
fn test_vcs_get_git_found() {
    let testdata = get_test_data_path("git-found");
    if let Err(e) = std::fs::create_dir_all(&testdata) {
        tracing::error!("failed to create dir: {}", e);
    }
    if let Err(e) = std::process::Command::new("git")
        .arg("init")
        .arg(".")
        .current_dir(&testdata)
        .output()
    {
        println!("testdata git dir not found: {e}");
    }

    let git = VcsClient::new_git(&testdata, None::<String>, None::<String>, "".into()).unwrap();
    assert_eq!(
        git,
        VcsClient::Git {
            source: testdata,
            username: "cuddle-please".into(),
            email: "bot@cuddle.sh".into(),
            token: "".into(),
        }
    )
}
